# # #
# /5/config.rb
# (c) 2009 Jan Lelis <mail@janlelis.de>. MIT License.
# See: https://rbjl.janlelis.com/5-playing-with-dijkstra
# # #

require 'yaml' # yet another markup language # comment out this line for JRuby 1.2.0 support (see readme!)

 # Config: global class/module which holds all configuration
 # access current program configuration by 'Config[:foo]' # => 'bar'
 # access general configuration (loaded by config file) as constants: 'Config::Foo' # => 'bar'
module Config
  @settings = {};
  def self.[](key); @settings[key]; end # config getter
  def self.[]=(key,val); # config setter
    raise ArgumentError, 'Sorry, the given algorithm is not suppported (see info!)'     if key == :algo   && ![:dijkstra,:astar,:astar_b,:backtracking].member?(val)
    raise ArgumentError, 'Sorry, the given output format is not suppported (see info!)' if key == :output && ![:console,:html,:both,:clean].member?(val)
    @settings[key] = val;
  end

  # This hash will hold program arguments..
  args = {}
  # ..get those now
  while arg = ARGV.shift
    unless arg[0]=='-' # param is no option
      arg = ARGV.shift # go to next argument
    else
      arg_val = '' # start with empty values..
      temp_val = ARGV.shift # (next param)
      while temp_val && temp_val[0]!='-'
        arg_val += ' ' + temp_val #..and add them to current the current arg-value
        temp_val = ARGV.shift # (next param)
      end
      args.merge_without_first!(arg,arg_val) # put options answers in hash
      ARGV.unshift(temp_val) # temp_val is next arg
    end
  end
  args.merge_without_first!(arg,arg_val) # put options answers in hash - also if end was already reached in loop

  # save allowed configuration
  args.each { |arg,arg_val| @settings[arg] = arg_val if [:file,:algo,:output,:force_menu,:info].member? arg}

  # load settings from config file
  config_path = (args[:config] || 'config/default.yaml').to_s
  base = YAML::load(File.read(config_path)) if File.file? config_path # comment out this line for JRuby 1.2.0 support (see readme!)
  # base = nil # remove comment from this line for JRuby 1.2.0 support (see readme!)

  if base and base['base_paths']
    BasePathLabs = base['base_paths']['labs']
    BasePathHtml = base['base_paths']['html']
  else
    BasePathLabs = 'labs/' # std
    BasePathHtml = 'html/' #   paths
  end
end
